use std::{collections::HashMap, fs::File, io::BufReader, path::Path};

use anyhow::Result;
use serde::{Deserialize, Serialize};
use serde_json::{from_reader, from_str};

pub fn load_keys<P: AsRef<Path> + std::fmt::Display>(
    path: Option<P>,
    verbose: bool,
) -> Result<Keys> {
    let keys = {
        if let Some(path) = path {
            if verbose {
                println!("Loading keys file {}", path);
            }

            let file = File::open(&path)?;
            let reader = BufReader::new(file);
            let keys: Keys = from_reader(reader)?;

            if verbose {
                println!("Loaded keys from {} successfully", path);
            }

            keys
        } else {
            from_str(std::include_str!("../../../keys.json"))?
        }
    };

    Ok(keys)
}

#[derive(Serialize, Deserialize)]
pub struct Keys {
    #[serde(rename = "Products")]
    pub products: HashMap<String, Product>,
    #[serde(rename = "BINK")]
    pub bink: HashMap<String, Bink>,
}

#[derive(Serialize, Deserialize)]
pub struct Product {
    #[serde(rename = "BINK")]
    pub bink: Vec<String>,
}

#[derive(Serialize, Deserialize)]
pub struct Bink {
    pub p: String,
    pub a: String,
    pub b: String,
    pub g: Point,
    #[serde(rename = "pub")]
    pub public: Point,
    pub n: String,
    #[serde(rename = "priv")]
    pub private: String,
}

#[derive(Serialize, Deserialize)]
pub struct Point {
    pub x: String,
    pub y: String,
}
